@@ -412,7 +412,8 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         """
         size_multiplier = self.renderer.size/100.
         plot_props = dict(plot_height=int(self.height*size_multiplier),
-                          plot_width=int(self.width*size_multiplier))
+                          plot_width=int(self.width*size_multiplier),
+                          sizing_mode=self.sizing_mode)
         if bokeh_version < '0.12':
             plot_props.update(self._title_properties(key, plot, element))
         if self.bgcolor:
@@ -1274,7 +1275,7 @@ class OverlayPlot(GenericOverlayPlot, LegendPlot):
                           'bgcolor', 'fontsize', 'invert_axes', 'show_frame',
                           'show_grid', 'logx', 'logy', 'xticks', 'toolbar',
                           'yticks', 'xrotation', 'yrotation', 'lod',
-                          'border', 'invert_xaxis', 'invert_yaxis']
+                          'border', 'invert_xaxis', 'invert_yaxis', 'sizing_mode']
 
     def _process_legend(self):
         plot = self.handles['plot']
@@ -1373,6 +1374,9 @@ class OverlayPlot(GenericOverlayPlot, LegendPlot):
             if tool:
                 renderers = tool.renderers+hover.renderers
                 tool.renderers = list(util.unique_iterator(renderers))
+            if 'hover' not in self.handles:
+                self.handles['hover'] = tool
+
 
     def _get_factors(self, overlay):
         xfactors, yfactors = [], []
