@@ -110,6 +110,8 @@ class BokehRenderer(Renderer):
         combining the bokeh model with another plot.
         """
         plot = super(BokehRenderer, self_or_cls).get_plot(obj, renderer)
+        if self_or_cls.mode == 'server' and doc is None:
+            doc = curdoc()
         if doc is not None:
             plot.document = doc
         return plot
@@ -140,7 +142,7 @@ class BokehRenderer(Renderer):
         default port ('localhost:8888').
         """
         renderer = self_or_cls.instance(mode='server')
-        # If show=False and not in noteboook context return document
+        # If show=False and not in notebook context return document
         if not show and not self_or_cls.notebook_context:
             doc, _ = renderer(plot)
             return doc
