@@ -458,8 +458,7 @@ class ElementPlot(GenericElementPlot, MPLPlot):
             return
 
         ranges = self.compute_ranges(self.hmap, key, ranges)
-        if not self.adjoined:
-            ranges = util.match_spec(element, ranges)
+        ranges = util.match_spec(element, ranges)
 
         label = element.label if self.show_legend else ''
         style = dict(label=label, zorder=self.zorder, **self.style[self.cyclic_index])
@@ -653,6 +652,13 @@ class ColorbarPlot(ElementPlot):
                 cs = np.array(cs)
             if len(cs) and cs.dtype.kind in 'if':
                 clim = ranges[vdim.name] if vdim.name in ranges else element.range(vdim)
+                if self.logz:
+                    # Lower clim must be >0 when logz=True
+                    # Choose the maximum between the lowest non-zero value
+                    # and the overall range
+                    if clim[0] == 0:
+                        vals = element.dimension_values(vdim)
+                        clim = (vals[vals!=0].min(), clim[1])
                 if self.symmetric:
                     clim = -np.abs(clim).max(), np.abs(clim).max()
             else:
@@ -660,7 +666,7 @@ class ColorbarPlot(ElementPlot):
         if self.logz:
             if self.symmetric:
                 norm = mpl_colors.SymLogNorm(vmin=clim[0], vmax=clim[1],
-                                         linthresh=clim[1]/np.e)
+                                             linthresh=clim[1]/np.e)
             else:
                 norm = mpl_colors.LogNorm(vmin=clim[0], vmax=clim[1])
             opts['norm'] = norm
