@@ -70,6 +70,7 @@ class Plot(param.Parameterized):
 
     @classmethod
     def lookup_options(cls, obj, group):
+        plot_class = None
         try:
             plot_class = Store.renderers[cls.backend].plotting_class(obj)
             style_opts = plot_class.style_opts
@@ -77,8 +78,10 @@ class Plot(param.Parameterized):
             style_opts = None
 
         node = Store.lookup_options(cls.backend, obj, group)
-        if group == 'style' and style_opts:
+        if group == 'style' and style_opts is not None:
             return node.filtered(style_opts)
+        elif group == 'plot' and plot_class:
+            return node.filtered(list(plot_class.params().keys()))
         else:
             return node
 
@@ -259,6 +262,8 @@ class DimensionedPlot(Plot):
         # Assumes composite objects are iterables
         if hasattr(self, 'subplots') and self.subplots:
             for el in self.subplots.values():
+                if el is None:
+                    continue
                 accumulator += el.traverse(fn, specs, full_breadth)
                 if not full_breadth: break
         return accumulator
@@ -580,7 +585,7 @@ class GenericElementPlot(DimensionedPlot):
 
         plot_element = self.hmap.last
         if self.batched and not isinstance(self, GenericOverlayPlot):
-            plot_element = [el for el in plot_element if el][-1]
+            plot_element = plot_element.last
 
         self.top_level = keys is None
         if self.top_level:
@@ -650,7 +655,7 @@ class GenericElementPlot(DimensionedPlot):
         frame = get_plot_frame(self.hmap, key_map, cached)
         traverse_setter(self, '_force', False)
 
-        if not key in self.keys:
+        if not key in self.keys and self.dynamic:
             self.keys.append(key)
         self.current_frame = frame
         self.current_key = key
@@ -722,17 +727,21 @@ class GenericElementPlot(DimensionedPlot):
         return xlabel, ylabel, zlabel
 
 
-    def _format_title(self, key, separator='\n'):
+    def _format_title(self, key, dimensions=True, separator='\n'):
         frame = self._get_frame(key)
         if frame is None: return None
         type_name = type(frame).__name__
         group = frame.group if frame.group != type_name else ''
         label = frame.label
 
-        dim_title = self._frame_title(key, separator=separator)
         if self.layout_dimensions:
+            dim_title = self._frame_title(key, separator=separator)
             title = dim_title
         else:
+            if dimensions:
+                dim_title = self._frame_title(key, separator=separator)
+            else:
+                dim_title = ''
             title_format = util.bytes_to_unicode(self.title_format)
             title = title_format.format(label=util.bytes_to_unicode(label),
                                         group=util.bytes_to_unicode(group),
@@ -992,8 +1001,8 @@ class GenericCompositePlot(DimensionedPlot):
         return len(self.keys)
 
 
-    def _format_title(self, key, separator='\n'):
-        dim_title = self._frame_title(key, 3, separator)
+    def _format_title(self, key, dimensions=True, separator='\n'):
+        dim_title = self._frame_title(key, 3, separator) if dimensions else ''
         layout = self.layout
         type_name = type(self.layout).__name__
         group = util.bytes_to_unicode(layout.group if layout.group != type_name else '')
